#!/usr/bin/perl

# cswpnchk.pl -- check weapon.pdt

$VERSION = "";

do "./common.pl";

if ($HELP) {
    &usage;
    exit 0;
}

# process the prog-specific args list passed on from common.pl
$a = @args = split "@", $ARGS;
for ($i=0; $i<$a; $i++) {
    print "$args[$i]\n" if $DEBUG;
    if ($HELP || ($args[$i] eq "-h") || ($args[$i] eq "-help")) {
        &usage;
        exit 0;
    } else {
        print "unrecognized option: $args[$i]\n\n";
        &usage;
        exit 1;
    } 
}

if (! ($ME || $VN || $CW || $EF || $WF || $PF)) {
    print "\nplease specify a game\n";
    &usage;
    exit 1;
}

&unitnames();  # in common.pl
&nofires();
&unitrangeeffects();

if ($DEBUG) {
    foreach $id (sort keys %unitnames) {
        print "UNITNAMES: $id $unitnames{$id}\n" if $DEBUG;
    }
    foreach $id (sort keys %unitrangeeffects) {
        print "UNITRANGEEFFECTS: $id $unitrangeeffects{$id}\n" if $DEBUG;
    }
    foreach $id (sort keys %nofires) {
        print "NOFIRES: $id $nofires{$id}\n" if $DEBUG;
    }
}

#exit 1;

# report units with missing range effect values
foreach $id (sort keys %unitnames) {

    $unm = lc($unitnames{$id});
    print "UNM: $id $unm\n" if $DEBUG;

    # categorize by platoon codes

    # PLATOON CODES 
    # 
    # August 28, 2016 
    # 
    # NOTE: This is for all games going forward, starting with Middle East. 
    # 
    # PXX0001 – PXX0999 Tanks, Light Tanks, Self-propelled Gun (as SU-100) 
    # PXX1001 – PXX1999 Artillery (Self-propelled and towed) 
    # PXX2001 – PXX2999 Infantry (All types) 
    # PXX3001 – PXX3199 Headquarters (All types) 
    # PXX3200 - PXX3999 Spare 
    # PXX4001 – PXX4199 Leaders (All types) 
    # PXX4200 - PXX4999 Spare 
    # PXX5001 – PXX5999 Off-map Airplanes (All types) 
    # PXX6001 – PXX6499 Reconnaissance Vehicles (Armoured Cars, Recce Jeeps, etc) 
    # PXX6500 – PXX6799 Helicopters and ON MAP aircraft (All types) 
    # PXX6800 – PXX6999 Naval Units (Boats, Landing Craft, etc) 
    # PXX7001 – PXX7999 Transport units (All types of Trucks, APC’s, IFV’s, etc) 
    # PXX8001 – PXX8499 Antiaircraft (Self-propelled, towed, man portable) 
    # PXX8500 – PXX8799 Rail 
    # PXX8800 – PXX8999 Misc Buildings, special units (Factories, News Crews, etc) 
    # PXX9001 – PXX9499 Antitank (Self-propelled, towed, man portable) 
    # PXX9500 – PXX9999 Spare 

    if (($id =~ /^\d\d[45]\d\d\d$/) || # leaders, off-map airplances,   ######
        ($id =~ /^\d[45]\d\d\d$/)   || # ditto,                         #####
        ($id =~ /^\d\d8[89]\d\d$/)  || # misc buildings, special units, ######
        ($id =~ /^\d8[89]\d\d$/)       # misc buildings, special units, #####
       ) {
        $type = "NOTICE";

    # categorize by no-fires, as in -firingcost is 0

    } elsif ($nofires{$id}) {
        $type = "NOTICE";

    # following, categorize by unit name

    # conveyances

    } elsif (index($unm, "transport") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "mover") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "truck") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "gmc") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "bus type") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "news car") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "omnibus") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "wagon") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "supplies") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "supply") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "tractor") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "limber") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "automobile") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "carrier") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "autocarri") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "dogsled") != -1) {
        $type = "NOTICE";

    } elsif (index($unm, "jeep") != -1) {
        $type = "NOTICE";  # WARNING?

    } elsif (index($unm, "bicycle") != -1) {
        $type = "WARNING";
    } elsif (index($unm, "motorcycle") != -1) {
        $type = "WARNING";

    } elsif (index($unm, "train") != -1) {
        $type = "WARNING";

    # water craft

    } elsif ((index($unm, "raft") != -1) && (index($unm, "aircraft") == -1)) {
        $type = "NOTICE";
    } elsif (index($unm, "ferry") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "boat") != -1) {
        $type = "WARNING";

    # animals

    } elsif (index($unm, "horse") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "mule") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "camel") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "pack animal") != -1) {
        $type = "NOTICE";

    # personnel

    } elsif (index($unm, "civilian") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "villager") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "personage") != -1) {
        $type = "NOTICE";

    } elsif (index($unm, "commander") != -1) {
        $type = "WARNING";
    } elsif (index($unm, "leader") != -1) {
        $type = "WARNING";

    # structures

    } elsif (index($unm, "depot") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "radar") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "cache") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "factory") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "warehouse") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "oil tank") != -1) {
        $type = "NOTICE";
    } elsif (index($unm, "refinery") != -1) {
        $type = "NOTICE";

    # default, for everything else (e.g., tanks, arty, etc.)

    } else {
        $type = "ERROR";
    }

    # by default, in all cases now ERROR
    # see:  http://www.matrixgames.com/forums/fb.asp?m=4343812
    $type = "ERROR";  # forced, in all cases

    if (! $unitrangeeffects{$id}) {
        if ($type eq "ERROR" ) {
            printf "%s: for unit %d %s, no corresponding range effect values in %s\n", $type, $id, &trim($unitnames{$id}), $weaponpdt if $SHOW_ERROR;
        } elsif ($type eq "WARNING" ) {
            printf "%s: for unit %d %s, no corresponding range effect values in %s\n", $type, $id, &trim($unitnames{$id}), $weaponpdt if $SHOW_WARNING;
        } elsif ($type eq "NOTICE" ) {
            printf "%s: for unit %d %s, no corresponding range effect values in %s\n", $type, $id, &trim($unitnames{$id}), $weaponpdt if $SHOW_NOTICE;
        }
    }
}

#exit 1;

# report unknown units found in weapon.pdt
foreach $id (sort keys %unitrangeeffects) {
    if (! $unitnames{$id}) {
        printf "NOTICE: in %s, unknown unit P%d\n", $weaponpdt, $id if $SHOW_NOTICE;
    }
}

#exit 1;

# report fire capable units with zero firing cost
foreach $id (sort keys %unitrangeeffects) {
    unless ($unitrangeeffects{$id} =~ /h\s+0\s+s\s+0/) {
        if ($nofires{$id}) {
            printf "ERROR: unit P%d with range effects %s has 0 fire cost\n", $id, $unitrangeeffects{$id} if $SHOW_ERROR;
        }
    }
}

#exit 1;

# report range effects that fail to uniformly decline with increasing range
foreach $id (sort keys %unitrangeeffects) {
    $e = @effects = split / /, $unitrangeeffects{$id};
    for ($i=0; $i<$e; $i++) {
        if (($effects[$i] eq "h") || ($effects[$i] eq "s")) {
                $prev = 0;
                next;
        }
        if (($effects[$i] > $prev) && ($prev > 0)) {
            printf "NOTICE: in %s, for unit %d, range effects don't uniformly decline: %s\n", $weaponpdt, $id, $unitrangeeffects{$id} if $SHOW_NOTICE;
            last;
        }
        $prev = $effects[$i];
    }
}

exit 0;

sub usage {
    print "\nUsage:  $progname [-h|-help] [{+|-}G|-debug] [{+|-}n] [{+|-}w] [{+|-}e] -g[ame] {me|vn|cw|ef|wf|pf}\n\n";
    return;
}

# read in weapon.pdt range effect values
sub unitrangeeffects {
    $weaponpdt = "weapon.pdt";
    # file existence checks are case insensitive
    if (! -e "$DATADIR/$weaponpdt") {
        printf "ERROR: $DATADIR/$weaponpdt not found!\n";
        exit 1;
    }
    open(WEAPONPDT, "$DATADIR/$weaponpdt");
    my $lineno = 0;
    while(<WEAPONPDT>) {
        chomp;
        $line = $_;
        $lineno++;
        #print "$line\n" if $DEBUG;
        if ($line =~ /^p(\d+)\s+([0-9][ 0-9]*[0-9]*)\s*$/i) {
            $id = $1;
            $unitrangeeffects{$id} = "h $2";
        } elsif ($line =~ /^([0-9][ 0-9]*)\s*$/) {
            $unitrangeeffects{$id} .= " s $1";
        } elsif ($line =~ /^\s*$/) {
            printf "ERROR: Empty line found in %s at line %d\n", $weaponpdt, $lineno if $SHOW_ERROR;
        } else {
            # Check for erroneous data input where a new data set starts without a newline
            if ($line =~ /([0-9][ 0-9]*)\s*p(\d+)\s+/i) {
                my $data = $1;
                my $new_id = $2;
                $unitrangeeffects{$id} .= " s $data";
                $unitrangeeffects{$new_id} = "h " . substr($line, index($line, $new_id) + length($new_id));
                printf "ERROR: Missing newline detected between data sets in %s at line %d\n", $weaponpdt, $lineno if $SHOW_ERROR;
            }
        }
    }
    close(WEAPONPDT);
}

# from csoobrpt.pl, read in no-fire weapons
sub nofires {
    open(NOFIRES, "$CSLINTROOT/csoobrpt.pl -g $game -id -firingcost -name |");
    #open(NOFIRES, "$CSLINTROOT/csoobrpt.pl -g $game -id -assault -firingcost -name |");
    while (<NOFIRES>) {
        chomp;
        $line = $_;
        print "$line\n" if $DEBUG;
        if ($line =~ /^p(\d+)\s+0 /i) {  # zero -firingcost
        #if ($line =~ /^p(\d+)\s+0 0 /i) {  # zero -assault & -firingcost
            $id = $1;
            $nofires{$id}++;
        }
    }
    close(NOFIRES);
}
